package spnego

import (
	"net/http"
	"strings"
	"fmt"
	// "github.com/MarshallWace/go-spnego"
)

// Transport extends the native http.Transport to provide SPNEGO communication
type Transport struct {
	GetCreds func(*http.Request) (string, string, string)
	ModifyRequest func(*http.Request)
	OnResponse func(*http.Request,*http.Response)
	
	Domain   string
	User     string
	Password string
	http.RoundTripper
	Jar http.CookieJar

	// spnego         Provider
	NoCanonicalize bool
}

// Error is used to distinguish errors from underlying libraries (gokrb5 or sspi).
type Error struct {
	Err error
}

// Error implements the error interface
func (e *Error) Error() string {
	return e.Err.Error()
}

// RoundTrip implements the RoundTripper interface.
func (t *Transport) RoundTrip(req *http.Request) (*http.Response, error) {
	fmt.Println("Trying Spnego Transport!")
	
	if t.GetCreds != nil {
		t.Domain, t.User, t.Password = t.GetCreds(req)
	}

	if t.Domain == "" && t.RoundTripper != nil {
		return t.RoundTripper.RoundTrip(req)
	}


	/*if t.spnego == nil {
		t.spnego = spnego.New()
	}

	if err := t.spnego.SetSPNEGOHeader(req, !t.NoCanonicalize); err != nil {
		return nil, &Error{Err: err}
	}*/

	res, err := t.RoundTripper.RoundTrip(req)
	// ToDo: process negotiate token from response
	if err != nil {
		return nil, err
	}

	if err == nil && res.StatusCode == http.StatusUnauthorized  {

		hasNegotiate := false
		authHeaders := res.Header.Values("WWW-Authenticate")
		for _, authHeader := range authHeaders {
			if authHeader == "Negotiate" {
				hasNegotiate = true
				break
			}
		}

		if hasNegotiate {
			
			if t.OnResponse != nil {
				t.OnResponse(req, res)
			}

			client := http.Client{}
			if t.RoundTripper != nil {
				client.Transport = t.RoundTripper
			}
		
			if t.Jar != nil {
				client.Jar = t.Jar
			}

			req2, _ := http.NewRequestWithContext(req.Context(),"GET", req.URL.String(), strings.NewReader(""))

			if t.ModifyRequest != nil {
				t.ModifyRequest(req2)
			}

			return client.Do(req2)
		}
	}

	return res, err
}